//******************************************************************************
// This class is part of the Sizmek Scratch and Reveal Ad Feature
// ALL RIGHTS RESERVED TO © 2018 Sizmek.
//******************************************************************************
//******************************************************************************
(function() {
	// Define Method
	window.ScratchAndReveal = function(_c) {

		"use strict";
		// Verify that Adkit
		if (document.location.protocol !== "file:" && (typeof EB === "undefined" || typeof EBG === "undefined")) {
			// Console error
			console.log('Adkit not ready');
			// abandon script
			return false;
		}
		// Save Reference to Script Name
		var scriptName = "Scratch_And_Reveal";
		// Script Version
		var scriptVersion = "4.0.0";
		// Last Modified
		var lastModified = "2018-06-15";
		// Last Uploaded
		var lastUploaded = "2018-06-15";
		// Template Version
		var templateVersion = "2.0.6";

		// Establish Config Object
		var config = _c;
		// Establish Boolean for tracking
		var isDebug = false;
		// Establish Boolean for tracking
		var trackOnce = true;
        // Establish reference to user agent
        var NAV_UA = navigator.userAgent;
        // Establish reference to canvas
        var CNVS;
        // Establish reference to canvas context
        var CTX;
        // Establish reference to canvas image
        var CNVS_IMG;
        // Establish reference to canvas unique id
        var CNVS_UID = Math.random().toFixed(10).substring(2);
        // Establish boolean for touch support
        var SUPPORTS_TOUCH  = "ontouchstart" in window || navigator.msMaxTouchPoints;
        // Establish boolean for mobile
        var IS_MOBILE       = (NAV_UA.match(/Mobi/i)) ? true : false;
        // Establish reference to percentage cleared
        var percentClear = 0;
        // Establish timers
        var uiTimer = null;
        var resizeTimer = null;
		// establish var for setTimeout
		var mouseOutTimeout;
        // Establish reference to current x/y position
        var currentX = 0;
        var currentY = 0;
        // Establish reference to prev x/y position
        var prevX = 0;
        var prevY = 0;
        // Establish boolean to track drawing status
        var isDrawing = false;
        // Establish reference for scale
        var resizedScale = 1;
        // Define required target div from config
        var TARGET_DIV_ID = _c.targetDivID;
        // Define required cover image from config
        var COVER = _c.cover;
        // Define optional brush size from config
    	var BRUSH_SIZE = _c.brushSize || 30;
        // Define optional clear % from config with default of 50
    	var PERCENT_CLEAR_TO_COMPLETE = _c.percentClearToComplete || 50;
        // Define optional auto complete time from config
    	var NO_UI_AUTO_COMPLETE_TIME = _c.noUIAutoCompleteTime || null;
        // Define optional auto complete timer after user interaction from config
    	var BREAK_IN_UI_AUTO_COMPLETE_TIME = _c.breakInUIAutoCompleteTime || null;
        // Define optional boolean for destroy div on complete
        var REMOVE_TARGET_DIV_ON_COMPLETE = (typeof _c.removeTargetDivOnComplete !== "undefined") ? _c.removeTargetDivOnComplete : true;
        // Define optional boolean to scale on resize
        var HANDLE_RESIZE = (typeof _c.handleResize !== "undefined") ? _c.handleResize : false;
        // Define callback for on ready
    	var ON_READY_CALL_BACK = _c.onReadyCallback || null;
        // Define callback for scratch start
    	var ON_UI_START_CALL_BACK = _c.onUIStartCallback || null;
        // Define callback for on scratch end
    	var ON_UI_END_CALL_BACK = _c.onUIEndCallback || null;
        // Define callback for complete
    	var ON_COMPLETE_CALL_BACK = _c.onCompleteCallback || null;
        // Establish reference to target div
        var TARGET_DIV = document.getElementById(TARGET_DIV_ID);
        // Determine initial display style of target div
        var TARGET_DIV_INITIAL_DISPLAY_STYLE = window.getComputedStyle(TARGET_DIV).getPropertyValue("display").replace(/["']/g, "").trim();
        // Determine width and height of target div
        var TARGET_DIV_INITIAL_WIDTH = TARGET_DIV.offsetWidth;
        var TARGET_DIV_INITIAL_HEIGHT = TARGET_DIV.offsetHeight;

		// Set up Script
		var init = function() {
            // Test if COVER is an image file URL - else treat as CSS color
            if (COVER.match(/(?:\.jpg)|(?:\.gif)|(?:\.png)|(?:\.svg)/g)){
                // Pre load image
                CNVS_IMG = new Image();
                // set origin
            	CNVS_IMG.crossOrigin = "anonymous";
                // place canvas on load
            	CNVS_IMG.onload = function(e){
                    placeCanvas();
            	};
                // Set source of image
            	CNVS_IMG.src = COVER;
            }
            else {
                // Set image to null
                CNVS_IMG = null;
                // place canvas
                placeCanvas();
            }
		};

        // Place canvas on page
        var placeCanvas = function(){
            // create new canvas element
            CNVS = document.createElement("canvas");
            // set canvas to width and height of target div
            CNVS.setAttribute("width", TARGET_DIV.offsetWidth);
            CNVS.setAttribute("height", TARGET_DIV.offsetHeight);
            // set canvas id
            CNVS.setAttribute("id", TARGET_DIV_ID + "_scrapeCanvas_" + CNVS_UID);
            // update styles
            CNVS.style.top  = "0px";
            CNVS.style.left = "0px";
            CNVS.style.position = "absolute";
            // Set canvas context
            CTX = CNVS.getContext("2d");
            // Place cover image
            placeCover(CNVS_IMG);
            // add canvas to target div
            TARGET_DIV.appendChild(CNVS);
            // add listeners
    		addEventListeners();
            // check if auto timer required and set if necessary
            if (typeof NO_UI_AUTO_COMPLETE_TIME === "number"){
                setTimmer(NO_UI_AUTO_COMPLETE_TIME);
            }
            // check if on ready callback required and add if necessary
            if (typeof ON_READY_CALL_BACK === "function") {
    			ON_READY_CALL_BACK();
    		}
        };

        // add listeners to canvas
        var addEventListeners = function(){
            // add touch listeners if supported
            if (SUPPORTS_TOUCH){
                CNVS.addEventListener("touchstart", onTouchStart, false);
        		CNVS.addEventListener("touchend", onTouchEnd, false);
        	}
            // if not supported add default listteners
            else {
        		CNVS.addEventListener("mousedown", onTouchStart, false);
        		CNVS.addEventListener("mouseup", onTouchEnd, false);
                // add mouseup to document instead of canvas
        		document.addEventListener("mouseup", onTouchEnd, false);
    	    }

			// Set listener to window so we kill draw on sustained mouseout
			CNVS.addEventListener("mouseout", function() {
				// on delay, fire touch end if still out of iframe
				mouseOutTimeout = setTimeout(function() {
					// stop drawing
					onTouchEnd();
				}, 500);
			}, false);

			// Listen for mouse in
			CNVS.addEventListener("mouseenter", function() {
					// Clear timeout
					clearTimeout(mouseOutTimeout);
				}
			);

            // check if resize handler required and add listener if necessary
            if (HANDLE_RESIZE === true){
                window.addEventListener("resize", onWindowResize, false);
            }
        };


        // DRAW
        var stampDraw = function(){
            // Save canvas context
        	CTX.save();
            // set context type
        	CTX.globalCompositeOperation = "destination-out";
            // Begin path
        	CTX.beginPath();
            // fill path
        	CTX.fillStyle = "rgba(0, 0, 0, 1)";
            // set line type to round
            CTX.lineCap = "round";
            // create arc
            CTX.arc(currentX, currentY, (BRUSH_SIZE * resizedScale)/2, 0, 2 * Math.PI, false);
            // fill
        	CTX.fill();
            // update ctx
        	CTX.restore();
        };

        // drag
        var dragDraw = function(){
            // save canvas context
        	CTX.save();
            // set context type
        	CTX.globalCompositeOperation = "destination-out";
            // begin path
        	CTX.beginPath();
            // move to prev x/y
        	CTX.moveTo(prevX, prevY);
            // move to current x/y
        	CTX.lineTo(currentX, currentY);
            // fill path
        	CTX.fillStyle = "rgba(0, 0, 0, 1)";
            // set line width
        	CTX.lineWidth = (BRUSH_SIZE * resizedScale);
            // set line type to round
            CTX.lineCap = "round";
            // complete line
        	CTX.stroke();
            // close path
        	CTX.closePath();
            // update context
        	CTX.restore();
        };

        // GET CORDS OF INPUT AT CANVAS
        var getAtCnvsCords = function(_x, _y){
            // Determine the canvas box dimensions
            var cnvsBox = CNVS.getBoundingClientRect();
            // Establish object to return new x/y
            var returnObj = {};

            // check for mobile and save new x/y
            if(IS_MOBILE === true){
                returnObj.x = (_x - cnvsBox.left-window.scrollX) * (CNVS.width/cnvsBox.width);
                returnObj.y = (_y - cnvsBox.top-window.scrollY) * (CNVS.height/cnvsBox.height);
    		}
            else {
                returnObj.x = (_x - cnvsBox.left) * (CNVS.width/cnvsBox.width);
                returnObj.y = (_y - cnvsBox.top) * (CNVS.height/cnvsBox.height);
    		}
            // return x/y
            return returnObj;
        };

        // Set up Timers
        var setTimmer = function(_t){
            uiTimer = window.setTimeout(onTimeOut, _t);
        };

        // Set up Timer for clearing canvas
        var clearUITimer = function(){
            // Check if timer exists
            if (uiTimer !== null){
                // clear timer
                window.clearTimeout(uiTimer);
                // set to null
                uiTimer = null;
            }
    	};

        // CALC PERCENT CLEARED
        var updatePercentCnvsCleared = function(){
            // Establish pixel count
    		var totalPxlAtAlphaZero = 0;
            // calculate total pixels
    		var imgData = CTX.getImageData(0, 0, CNVS.width, CNVS.height);
            var pxlData = imgData.data;
            var totalPxls = (pxlData.length/4);

            // loop through pixels
            for(var i = 3; i <= pxlData.length; i+=4){
                // check if pixel is clear
    			if(pxlData[i] <= 0){
                    // increment pixel count
    				totalPxlAtAlphaZero++;
    			}
    		}
            // get % clear
    		percentClear = ((totalPxlAtAlphaZero / totalPxls) * 100).toFixed(2);

            // determie if cleared is greater than clear pref
    		if (typeof PERCENT_CLEAR_TO_COMPLETE === "number"){
            	if(percentClear >= PERCENT_CLEAR_TO_COMPLETE){
                    // copmlete
            		complete();
    			}
    		}
    	};

        // COMPLETE scratch
        var complete = function(){
            // Clear timer
            clearUITimer();
            // if necessary, destroy div
            if(REMOVE_TARGET_DIV_ON_COMPLETE === true){
                removeTargetDiv();
            }
            // if necessary fire on complete callback
            if (typeof ON_COMPLETE_CALL_BACK === "function"){
                ON_COMPLETE_CALL_BACK();
            }
        };

        // RESET / RESTORE scratch
        var restoreInitialState = function(){
            // clear timer
            clearUITimer();
            // replace target div
            replaceTargetDiv();
            // Clear canvas context
            CTX.clearRect(0, 0, CNVS.width, CNVS.height);
            // reset clear count
            percentClear = 0;
            // replace image
            placeCover(CNVS_IMG);

            // set auto timer if necessary
            if (typeof NO_UI_AUTO_COMPLETE_TIME === "number"){
                setTimmer(NO_UI_AUTO_COMPLETE_TIME);
            }
            // call on ready callback if necessary
            if(typeof ON_READY_CALL_BACK === "function") {
    			ON_READY_CALL_BACK();
    		}
        };

        // RESIZE
        var resizeCnvs = function(){
            // Establish reference to canvas image data
            var cnvsImgData;
            // Estable reference to new image
            var cnvsAsImg;
            // reset timer
            resizeTimer = null;
            // determine new scale
            updateResizedScale();

            // Check if canvas is new
            if (percentClear === 0){
                // set size
                setCnvsSize();
                // Clear canvas context
                CTX.clearRect(0, 0, CNVS.width, CNVS.height);
                // place new image
                placeCover(CNVS_IMG);
            }
            else {
                // get image data
                cnvsImgData = CNVS.toDataURL();
                // Create new image
                cnvsAsImg = new Image();
                // set origin
            	cnvsAsImg.crossOrigin = "anonymous";
                // add listener
                cnvsAsImg.onload = function(e){
                    // set canvas size on load
                    setCnvsSize();
                    // clear canvas on load
                    CTX.clearRect(0, 0, CNVS.width, CNVS.height);
                    // place image on load
                    placeCover(cnvsAsImg);
                };
                // update source
                cnvsAsImg.src = cnvsImgData;
            }

            // TODO: show content here
        };

        // Set Canvas height and width
        var setCnvsSize = function(){
            CNVS.setAttribute("width", TARGET_DIV.offsetWidth);
            CNVS.setAttribute("height", TARGET_DIV.offsetHeight);
        };

        // Get new image scale
        var updateResizedScale = function(){
            // NOTE: Uncomment the below if you want to get the scale which is required to set a cover image to replicate the CSS background cover attribute.
            // var initialRatio = (TARGET_DIV_INITIAL_HEIGHT / TARGET_DIV_INITIAL_WIDTH);
            // var currentRatio = (TARGET_DIV.offsetHeight / TARGET_DIV.offsetWidth);
            // var scale = (currentRatio > initialRatio) ? (TARGET_DIV.offsetHeight  / TARGET_DIV_INITIAL_HEIGHT) : (TARGET_DIV.offsetWidth / TARGET_DIV_INITIAL_WIDTH) ;
            //
            // resizedScale = scale;

            resizedScale = (TARGET_DIV.offsetHeight / TARGET_DIV_INITIAL_HEIGHT);
        };

        // PLACE COVER
        var placeCover = function(_img){
            // Verify image and draw on canvas
            if (_img !== null){
                CTX.drawImage(_img, 0, 0, CNVS.width, CNVS.height);
            }
            else {
                // Save canvas context
                CTX.save();
                // fill with image
                CTX.fillStyle = COVER;
                // fill rect
                CTX.fillRect(0, 0, CNVS.width, CNVS.height);
                // update canvas context
                CTX.restore();
            }
        };

        // ADD & REMOVE TARGET DIV
        var removeTargetDiv = function(){
            TARGET_DIV.style.display = "none";
        };

        // update target div style
        var replaceTargetDiv = function(){
            TARGET_DIV.style.display = TARGET_DIV_INITIAL_DISPLAY_STYLE.toString();
        };

        /* ------------------------------------ */
        /* -----> PRIVATE EVENT LISTNERS <----- */
        /* ------------------------------------ */

        /* -----> TIMER COMPLETE <----- */
        var onTimeOut = function(e){
            complete();
        };

        /* -----> TOUCH LISTNERS <----- */
        var onTouchStart = function(e){
            e.preventDefault();

			if (!isDrawing){
	            // save touch event
	        	var data = (SUPPORTS_TOUCH === true) ? e.changedTouches[0] : e;
	            // get x/y
	            var atCnvsCords = getAtCnvsCords(data.clientX, data.clientY);
	            // clear timer
	            clearUITimer();

	            // verify touch support and add appropriate listeners
	            if (SUPPORTS_TOUCH){
	        		CNVS.addEventListener("touchmove", onTouchMove, false);
	        	}
	            else{
	        		CNVS.addEventListener("mousemove", onTouchMove, false);
	        		document.addEventListener("mousemove", onTouchMove, false);
	        	}

	            // Update X/Y
	            prevX = currentX;
	        	prevY = currentY;
	        	currentX = atCnvsCords.x;
	        	currentY = atCnvsCords.y;
	            // Update boolean
	            isDrawing = true;
	            // Draw
	            stampDraw();

	            // call UI start callback if necessary
	            if (typeof ON_UI_START_CALL_BACK === "function") {
	    			ON_UI_START_CALL_BACK();
	    		}
			}
        };

        // Handle on touch move
        var onTouchMove = function(e){
            e.preventDefault();
            // save touch event
        	var data = (SUPPORTS_TOUCH === true) ? e.touches[0] : e;
            // save x/y
            var atCnvsCords = getAtCnvsCords(data.clientX, data.clientY);
            // update x/y
            prevX = currentX;
        	prevY = currentY;
        	currentX = atCnvsCords.x;
        	currentY = atCnvsCords.y;
            // Draw
            dragDraw();
        };

        // handle touch end event
        var onTouchEnd = function(e){
            // Save touch event data
            var data = (SUPPORTS_TOUCH === true) ? e.touches[0] : e;
            // save x/y
            var atCnvsCords = {};
			try {
				atCnvsCords = getAtCnvsCords(data.clientX, data.clientY);
			}
			catch (d) {
				atCnvsCords.x = prevX;
				atCnvsCords.y = prevY;
			}

            // stop if not drawing
            if (isDrawing === false){
                return;
            }

            // verify touch support and add appropriate listeners
        	if (SUPPORTS_TOUCH){
        		CNVS.removeEventListener("touchmove", onTouchMove, false);
        	}
            else {
        		CNVS.removeEventListener("mousemove", onTouchMove, false);
        		document.removeEventListener("mousemove", onTouchMove, false);
        	}

            // update x/y
            prevX = currentX;
        	prevY = currentY;
        	currentX = atCnvsCords.x || prevX;
        	currentY = atCnvsCords.y || prevY;
            // update boolean
        	isDrawing = false;

            // call UI end callback if necessary
            if (typeof ON_UI_END_CALL_BACK === "function") {
    			ON_UI_END_CALL_BACK();
    		}

            // set timer if necessary
            if (typeof BREAK_IN_UI_AUTO_COMPLETE_TIME === "number"){
                setTimmer(BREAK_IN_UI_AUTO_COMPLETE_TIME);
            }
            // update % cleared
            updatePercentCnvsCleared();
        };

        // Handle resize
        var onWindowResize = function(e){
            // check if div has been resized
            if (TARGET_DIV_INITIAL_HEIGHT !== TARGET_DIV.offsetHeight || TARGET_DIV_INITIAL_WIDTH !== TARGET_DIV.offsetWidth){
                // TODO: hide content here

                // clear timeout if timer exists
                if (resizeTimer !== null){
                    window.clearTimeout(resizeTimer);
                }
                // start resize timer
                resizeTimer = window.setTimeout(resizeCnvs, 300);
            }
        };

        // restore canvas image
        var restore = function(){
    		restoreInitialState();
    	};

        // return Canvas ID
    	var getCanvasID = function(){
    		return CNVS.id;
    	};

        // return % cleared
    	var getPercentClear = function(){
    		return percentClear;
    	};

        // Clear canvas
    	var forceComplete = function(){
    		complete();
    	};

        // resize canvas
    	var forceResize = function(){
    		resizeCnvs();
    	};

    	// This should be fired before clickout or panel close
    	var clearAutoCompleteTimer = function(){
    		clearUITimer();
    	};

		// DO NOT REMOVE THE FOLLOWING
		//=================================================
		// Utility Functions
		//-------------------------------------------------

		// Handle Dispatching events
		var dispatchEvent = function(_name, _detail) {
			// Establish var for event
			var e = document.createEvent("Event");
			// Init Event
			e.initEvent(_name, true, false);
			// Pass Success Message
			e.detail = _detail;
			// Dispatch Event
			window.dispatchEvent(e);
		};

		// Debugging
		var log = function() { // this is a closure-compiled version of the original code
			if (isDebug) {
				var b, a;
				b = Array.prototype.slice.call(arguments);
				a = new Date();
				a = scriptName + " (" + a.getFullYear() + "-" + a.getMonth() + "-" + a.getDate() + " " + (("0" + a.getHours()).slice(-2)) + ":" + (("0" + a.getMinutes()).slice(-2)) + ":" + (("0" + a.getSeconds()).slice(-2)) + "." + (("00" + a.getMilliseconds()).slice(-3)) + "): ";
				b.unshift(a);
				try {
					window.console && console.log && (console.log.apply ? console.log.apply(console, b) : console.log(b));
				}
				catch (d) {}
			}
		};

		// Track Ad Features - accepts interaction name and impression type
		var trackAdFeatures = function(_interaction, _noun) {
			// Establish var to track local mode
			var isServed;
			// Verify ad is not local
			try {
				isServed = EB._isServingMode();
			}
			catch (err) {}

			if (isServed && trackOnce) {
				try {
					// Update Boolean
					trackOnce = false;
					// Grab Version ID
					var adId = EB._adConfig.adId;
					// Establish reference for Version id
					var versionID;
					// Grab Version ID for mass versioning
					try {
						versionID = EB._adConfig.massVersioning.adVersions.split("_")[0];
					}
					catch (err) {
						versionID = "";
					}
					// Determine if tag is SAS
					var isSAS = (adId.toString().length >= 10);
					// Build Base Url for SAS and MDX
					var baseURL = (isSAS) ? "https://bs.serving-sys.com/Serving/adServer.bs?cn=display&c=19&pli=1073952795&adid=1073972634&ord=[timestamp]&rtu=-1&pcp=$$adID=" + adId + "|vID=" + versionID + "|interactionName=" : "https://bs.serving-sys.com/BurstingPipe/adServer.bs?cn=tf&c=19&mc=imp&pli=16478984&PluID=0&ord=%time%&rtu=-1&pcp=$$adID=" + adId + "|vId=" + versionID + "|interactionName=";
					// Build Tracking Pixel
					var trackingPixel = baseURL + _interaction + "|noun=" + _noun + "$$";
					// Fire Tracking Pixel by creating a new image.
					new Image().src = trackingPixel;

				}
				catch (err) {}
			}
		};

		// Set up Script
		var prepareFeature = function() {
			// Track Ad Feature
			trackAdFeatures("impression", "AF_" + scriptName);
			// init script
			init();
		};

		// set up script
		prepareFeature();

		// Return functions you want publically available
		return {
			restore: restore,
			getCanvasID: getCanvasID,
            getPercentClear: getPercentClear,
            forceResize: forceResize,
            clearAutoCompleteTimer: clearAutoCompleteTimer,
            complete: complete,
		};
	};
}());
